import { ElementRef, EventEmitter, AfterViewInit } from '@angular/core';
import { Observable, Subscription } from 'rxjs';
export declare class NgxCameraComponent implements AfterViewInit {
    /** default video options */
    static DEFAULT_VIDEO_OPTIONS: MediaTrackConstraints;
    /** last captured image data */
    captureImageData: any;
    /** canvas clear subscription  */
    clearSubscription: Subscription;
    /** camara trigger subscription  */
    triggerSubscription: Subscription;
    /** camara switch subscription  */
    cameraSwitcherSubscription: Subscription;
    /** event emitter for captured image  */
    mediaStream: any;
    /** refrence for video player tag  */
    private video;
    /** refrence for canvas object tag */
    private canvas;
    /** camera area width */
    width: number;
    /** camera area height */
    height: number;
    /** define camera image type */
    imageType: CameraImageType | string;
    /** should start camera in after view init or not */
    startCam: boolean;
    /** define camera image quality */
    imageQuality: number;
    /** event emitter for detected devices */
    detectedDevices: EventEmitter<MediaDeviceInfo[]>;
    /** event emitter for captured video stream */
    stream: EventEmitter<MediaDeviceInfo[]>;
    /** event emitter for errors occord at diffrent stages */
    errorStream: EventEmitter<any>;
    /** event emitter for errors occord at diffrent stages */
    imageCapture: EventEmitter<CameraImage>;
    /**
     * If the given Observable emits, clear canvas
     */
    clear: Observable<void>;
    /**
     * If the given Observable emits, an image will be captured and emitted through 'imageCapture' EventEmitter
     */
    trigger: Observable<void>;
    /**
     * If the given Observable emits, the active webcam will be switched to the one indicated by the emitted value.
     * @param switchCamera Indicates which webcam to switch to
     *   MediaDeviceInfo: activate the webcam with the given  MediaDeviceInfos
     */
    switchCamera: Observable<MediaDeviceInfo>;
    /**
     * get native html element from video elementeRef
     * @returns native html element from video elementeRef
     */
    readonly VideoElement: any;
    /** available camera inputs list */
    availableCameraInputs: MediaDeviceInfo[];
    /** if video os initialized or not */
    videoInitialized: boolean;
    /**
     * method to return MediaTrackConstraints from device id and baseMediaTrackConstraints
     * @param deviceId device id to merge MediaTrackConstraints with. you can pass null if you just want to get MediaTrackConstraints.
     * @param baseMediaTrackConstraints MediaTrackConstraints to merge with or you can use this getter ot get default MediaTrackConstraints bu passing null.
     * @returns merged MediaTrackConstraints from given device id and baseMediaTrackConstraints
     */
    private static getMediaConstraintsForDevice;
    constructor();
    ngAfterViewInit(): void;
    /**
     * allows you to switch to a spicific camera
     * @param info media info of device that you want to switch camera to
     */
    switchToVideoInput(info: MediaDeviceInfo): void;
    /**
     * start a camera with particular device id
     * @param deviceId device id to start camera on
     */
    startCamera(deviceId: string): void;
    /**
     * list all the detected media device with kind video input
     * @returns observable of detected media devices after promis is resolved
     */
    private detectAvailableDevices;
    /**
     *  initinitialize web cam with device id and constraints and assign all the data streams to respective variables.
     * @param deviceId device id to initinitialize web cam on
     * @param userVideoTrackConstraints user video track constraints.
     */
    private initWebcam;
    /**
     * stops all the media tracks
     */
    stopMediaTracks(): void;
    /**
     * Takes a snapshot of the current webcam's view and emits the image as an event
     */
    takeSnapshot(): void;
}
export interface CameraConfigInterface {
    width: number;
    height: number;
    element: ElementRef;
    imageType: CameraImageType;
    imageQuality: number;
}
export declare enum CameraImageType {
    jpeg = "image/jpeg",
    png = "image/png"
}
export declare class CameraImage {
    constructor(imageAsDataUrl: string, mimeType: string, imageData: ImageData);
    private readonly _mimeType;
    private _imageAsBase64;
    private readonly _imageAsDataUrl;
    private readonly _imageData;
    /**
     * Extracts the Base64 data out of the given dataUrl.
     * @param dataUrl the given dataUrl
     * @param mimeType the mimeType of the data
     */
    private static getDataFromDataUrl;
    /**
     * Get the base64 encoded image data
     * @returns base64 data of the image
     */
    readonly imageAsBase64: string;
    /**
     * Get the encoded image as dataUrl
     * @returns the dataUrl of the image
     */
    readonly imageAsDataUrl: string;
    /**
     * Get the ImageData object associated with the canvas' 2d context.
     * @returns the ImageData of the canvas's 2d context.
     */
    readonly imageData: ImageData;
}
